# TSCloud.py
# Version 1.2, Feb. 21, 2019

from tcpcom import *
import time
import gc

_host = "api.thingspeak.com"
_port = 443
_maxEntries = 8

class ThingSpeakChannel:
    def __init__(self, channelId, readKey, writeKey, userKey, verbose = False):
        self._channelId = channelId
        self._readKey = readKey
        self._writeKey = writeKey
        self._userKey = userKey
        self._labels = []
        self._isVerbose = verbose

    def getData(self, nbEntries = -1, includeDate = True):
        gc.collect()
        inquiry = "/channels/" + str(self._channelId) + \
            "/feeds.json?api_key=" + self._readKey + "&results=" + (str(nbEntries) if nbEntries != -1 else "8000")
        self._debug("GET inguiry: " + inquiry)
        client = HTTPClient()
        client.connect(_host, _port)
        reply = client.sendGetRequest(inquiry)
        client.closeConnection()
        self._debug("reply: " + reply)
        if HTTPClient.getHeader(reply).find('400 Bad Request') != -1:
            self._debug("Bad request")
            return None            
        dataDict = HTTPClient.extractJSON(reply)
        if dataDict == None:
            return []
        self._debug("dict: " + str(dataDict))
        self._labels.clear()
        for label in range(1, _maxEntries + 1):
            try:
                dataDict['channel']['field' + str(label)]
                self._labels.append(label)
            except:
                pass    
    
        self._debug("labels: " + str(self._labels))
        
        tableList = []
        nbFeeds = len(dataDict['feeds'])
        self._debug("nbFeeds: " + str(nbFeeds))
        for entryId in range(0, nbFeeds):
            values = {}
            for label in self._labels:
                try:
                    value = str(dataDict['feeds'][entryId]['field' + str(label)])
                    if includeDate:
                        date = str(dataDict['feeds'][entryId]['created_at'])
                        values['date'] = date.replace('T', ' ').replace('Z', '')
                    values[label] = value
                except:
                    pass
            if values == {}:
                break    
            tableList.append(values)
        self._debug("tableList: " + str(tableList))
        return tableList
    
    def getLastValue(self, fieldId = 1):
        lastData = self.getData(1)
        if lastData == None or lastData == []: #empty channel
            return None
        return lastData[0][fieldId]

    def getFields(self):
        inquiry = "/channels/" + str(self._channelId) + \
            "/feeds.json?api_key=" + self._readKey + "&results=1"
        client = HTTPClient()
        client.connect(_host, _port)
        reply = client.sendGetRequest(inquiry)
        client.closeConnection()
        if HTTPClient.getHeader(reply).find('400 Bad Request') != -1:
            self._debug("Bad request")
            return None           
        dataDict = HTTPClient.extractJSON(reply)
        fieldNames = {}
        for label in range(1, _maxEntries + 1):
            try:
                key = 'field' + str(label)
                value = dataDict['channel'][key]
                fieldNames[key] = value
            except:
                pass    
        return fieldNames    
    
    def insert(self, data, timeout = 20):
        if type(data) == dict:
            self._debug("inserting from dictionary: " + str(data))
            query = ""
            for key in data.keys():
                query = query + "&field" + str(key) + "=" + str(data[key])
                inquiry = "/update/?api_key=" + self._writeKey + query
        else:
            self._debug("inserting single value: " + str(data))
            inquiry = "/update/?api_key=" + self._writeKey + "&field1=" + str(data)
        nbTrials = 0
        while nbTrials < int(timeout):
            rc = self._insert(inquiry)
            if rc == -1:
                return True
            elif rc == 1:
                return False
            nbTrials += 1
            self._debug("insertion failed. nTrials: " + str(nbTrials))
            time.sleep(1)
        return False    
    
    def _insert(self, inquiry):             
        try:
            self._debug("inquiry: " + inquiry)
            client = HTTPClient()
            client.connect(_host, _port)
            reply = client.sendGetRequest(inquiry)
            client.closeConnection()
            self._debug("reply: " + reply)
            if HTTPClient.getHeader(reply).find('400 Bad Request') != -1:
                return 1            
            if HTTPClient.getContent(reply) == '0':
                return 0
        except:
            return 1
        return -1

    def clear(self):
        inquiry = "/channels/" + str(self._channelId) + "/feeds.json?api_key=" + self._userKey
        self._debug("clearChannel inquiry: " + inquiry)
        client = HTTPClient()
        client.connect(_host, _port)
        reply = client.sendDeleteRequest(inquiry)
        client.closeConnection()
        if HTTPClient.getHeader(reply).find('200 OK') != -1:
            return True            
        return False
        
    def _debug(self, msg):
        if self._isVerbose:
            print("---> Debug:" + str(msg))   
