# drv2605.py
# Haptic Motor
# Adapted from Seed driver with thanks to the author
# Version 2.00 - Aug 14, 2018

from utils import I2C_com
from machine import Pin
import time

class DRV2605():
    _STATUS_REG =         const(0x00)
    _MODE_REG =           const(0x01)
    _RTP_INPUT_REG =      const(0x02)
    _LIB_SEL_REG =        const(0x03)
    _WAV_SEQ1_REG =       const(0x04)
    _WAV_SEQ2_REG =       const(0x05)
    _WAV_SEQ3_REG =       const(0x06)
    _WAV_SEQ4_REG =       const(0x07)
    _WAV_SEQ5_REG =       const(0x08)
    _WAV_SEQ6_REG =       const(0x09)
    _WAV_SEQ7_REG =       const(0x0A)
    _WAV_SEQ8_REG =       const(0x0B)
    _GO_REG =             const(0x0C)
    _ODT_OFFSET_REG =     const(0x0D)
    _SPT_REG =            const(0x0E)
    _SNT_REG =            const(0x0F)
    _BRT_REG =            const(0x10)
    _ATV_CON_REG =        const(0x11)
    _ATV_MIN_IN_REG =     const(0x12)
    _ATV_MAX_IN_REG =     const(0x13)
    _ATV_MIN_OUT_REG =    const(0x14)
    _ATV_MAX_OUT_REG =    const(0x15)
    _RATED_VOLTAGE_REG =  const(0x16)
    _OD_CLAMP_REG =       const(0x17)
    _A_CAL_COMP_REG =     const(0x18)
    _A_CAL_BEMF_REG =     const(0x19)
    _FB_CON_REG =         const(0x1A)
    _CONTRL1_REG =        const(0x1B)
    _CONTRL2_REG =        const(0x1C)
    _CONTRL3_REG =        const(0x1D)
    _CONTRL4_REG =        const(0x1E)
    _VBAT_MON_REG =       const(0x21)
    _LRA_RESON_REG =      const(0x22)
    
    HAPTIC_SHORT = 1
    HAPTIC_LONG = 15
    HAPTIC_VERY_LONG = 16
    HAPTIC_ULTRA_LONG = 118
    HAPTIC_TWO = 10
    HAPTIC_THREE = 12
    HAPTIC_RATTLE1 = 52
    HAPTIC_RATTLE2 = 54
    
    HAPTIC_ALL = [HAPTIC_SHORT, HAPTIC_LONG, HAPTIC_VERY_LONG, 
                 HAPTIC_TWO, HAPTIC_THREE, HAPTIC_RATTLE1, 
                 HAPTIC_RATTLE2, HAPTIC_ULTRA_LONG]    

    # ---------------------- ctor ---------------------
    def __init__(self, scl = Pin(22), sda = Pin(21), addr = 0x5A):
        self._i2c_com = I2C_com(addr, scl, sda)
        self._setup()

    def getDeviceID(self):
       return self._i2c_com.readReg(_STATUS_REG)
    
    def _setup(self):
        # Enter standby mode
        self._i2c_com.writeReg(_MODE_REG, 0x40)
        time.sleep(0.001)
    
        # Set rate voltage
        self._i2c_com.writeReg(_RATED_VOLTAGE_REG, 0x50)
    
        # Set overdrive voltage
        self._i2c_com.writeReg(_OD_CLAMP_REG, 0x89)
        
        # Setup feedback control and control registers 
        self._i2c_com.writeReg(_FB_CON_REG, 0xB6)
        self._i2c_com.writeReg(_CONTRL1_REG, 0x13)
        self._i2c_com.writeReg(_CONTRL2_REG, 0xF5)
        self._i2c_com.writeReg(_CONTRL3_REG, 0x80)
    
        # Select the LRA Library 
        self._i2c_com.writeReg(_LIB_SEL_REG, 0x06)
    
        # Put the DRV2605 device in active mode 
        self._i2c_com.writeReg(_MODE_REG, 0x00)
    
    def playWaveform(self, n):
        self._i2c_com.writeReg(_WAV_SEQ1_REG, n)
        self._i2c_com.writeReg(_GO_REG, 0x01)


