# ds3231.py
# Support for the DS3231 based Realtime Clock
# Code adapted from MultiWingSpan/M.Atkinson, www.multiwingspan.co.uk

from machine import I2C, Pin
from time import sleep

class DS3231:
    def __init__(self, clk = Pin(22), dta = Pin(21)):
        self.addr = 0x68
        self.i2c = I2C(scl = clk, sda = dta)
        self.buf = bytearray(7)
    
    def bcd2dec(self, bcd):
        return (((bcd & 0xf0) >> 4) * 10 + (bcd & 0x0f))

    def dec2bcd(self, dec):
        tens, units = divmod(dec, 10)
        return (tens << 4) + units
    
    def getDateTime(self):
        # returns year, month, day, hours, minutes, seconds, dayOfWeek (0: monday, arbitrary)
        self.i2c.writeto(self.addr, b'\x00')
        self.buf = self.i2c.readfrom(self.addr, 7)
        s = self.bcd2dec(self.buf[0])
        m = self.bcd2dec(self.buf[1])
        if self.buf[2] & 0x40:
            h = self.bcd2dec(self.buf[2] & 0x1f)
            if self.buf[2] & 0x20:
                h += 12
        else:
            h = self.bcd2dec(self.buf[2])
        w = self.buf[3]
        dd = self.bcd2dec(self.buf[4])
        mm = self.bcd2dec(self.buf[5] & 0x1f)
        yy = self.bcd2dec(self.buf[6]) + 2000
        return yy, mm, dd, h, m, s, w
    
    def setDateTime(self, yy, mm, dd, h, m, s, w):
        # sets year, month, day, hours, minutes, seconds, dayOfWeek (0: monday)
        t = bytes([s, m, h, w, dd, mm, yy - 2000])
        for i in range(0,7):
            self.i2c.writeto(self.addr, bytes([i, self.dec2bcd(t[i])]))
        sleep(1)
