# minimotor.py
# Version 1.00 - Aug 14, 2018

from utils import I2C_com
from machine import Pin

# ----------------------------------- class MiniMotor -------------------------
'''
 Send the drive command over I2C at reg 0x00 to the DRV8830 chip. Bits 7:2 are the speed
 setting; range is 0-63. Bits 1:0 are the mode setting:
  - 00 = HI-Z
  - 01 = Reverse
  - 10 = Forward
  - 11 = H-H (brake)
'''


class MiniMotor:
    _REGISTER_DIRECTION = const(0x00)
    _REGISTER_STATUS = const(0x01)
    
    def __init__(self, scl = Pin(22), sda = Pin(21), addr = 0x60):
        self._i2c_com = I2C_com(addr, scl, sda)

    def getFault(slef):
        return self._i2c_com.readReg(_REGISTER_STATUS)
    
    def clearFault(self):
        self._i2c_com.writeReg(_REGISTER_STATUS, 0x80)
        
    def setSpeed(self, speed):                                                                            
        self.speed = speed
    
    def rotate(self, speed):
        self.clearFault()
        value = abs(speed)
        if value > 63:
            value = 63
        value = value << 2
        if speed < 0:
            value |= 0x01    
        else:
            value |= 0x02
        self._i2c_com.writeReg(_REGISTER_DIRECTION, value)                                    
    
    def stop(self):
        self._i2c_com.writeReg(_REGISTER_DIRECTION, 0)                                    


motA = MiniMotor(addr = 0x60)
motB = MiniMotor(addr = 0x62)

# ----------------------------------- class OxoRover -------------------------
class OxoRover:
    def __init__(self, speed = 63):
        self.speed = speed
        
    def forward(self):
        motA.rotate(self.speed)    
        motB.rotate(self.speed)    

    def backward(self):
        motA.rotate(-self.speed)    
        motB.rotate(-self.speed)    
        
    def left(self):
        motA.rotate(-self.speed)    
        motB.rotate(self.speed)    

    def leftArc(self):
        motA.rotate(self.speed // 2)    
        motB.rotate(self.speed)    
    
    def right(self):
        motA.rotate(self.speed)    
        motB.rotate(-self.speed)    

    def rightArc(self):
        motA.rotate(self.speed)    
        motB.rotate(self.speed // 2)    

    def stop(self):
        motA.stop()    
        motB.stop()


