# mpl3115a2.py
# Version 1.15 - Sep 15, 2018
# Code from  ControlEverything.com
# Adapted for ESP32 by Aegidius Pluess, www.aplu.ch

import time

from utils import I2C_com
from machine import Pin

class MPL3115A2:
    def __init__(self, i2c_address = 0x60, scl = Pin(22), sda = Pin(21)):
        self.i2c = I2C_com(i2c_address, scl, sda)
        # Resetting
        self.i2c.writeReg(0x26, 0x04)
        time.sleep(1)
        # Select control register, 0x26(38)
        #        0xB9(185)   Active mode, OSR = 128, Altimeter mode
        
    def getValues(self, seaLevel = None):
        self.i2c.writeReg(0x26, 0xB9)
        # Select data configuration register, 0x13(19)
        #       0x07(07)    Data ready event enabled for altitude, pressure, temperature
        if seaLevel != None:
            self._setSeaLevel(seaLevel)
        self.i2c.writeReg(0x13, 0x07)
        # Select control register, 0x26(38)
        #       0xB9(185)   Active mode, OSR = 128, Altimeter mode
        self.i2c.writeReg(0x26, 0xB9)
        time.sleep(1)
        # Read data back from 0x00(00), 6 bytes
        # status, tHeight MSB1, tHeight MSB, tHeight LSB, temp MSB, temp LSB
        data = self.i2c.readBlock(0x00, 6)
        
        # Convert the data to 20-bits
        tHeight = ((data[1] * 65536) + (data[2] * 256) + (data[3] & 0xF0)) / 16
        temp = ((data[4] * 256) + (data[5] & 0xF0)) / 16
        altitude = tHeight / 16.0
        cTemp = temp / 16.0
        
        # Select control register, 0x26(38)
        #       0x39(57)    Active mode, OSR = 128, Barometer mode
        self.i2c.writeReg(0x26, 0x39)
        time.sleep(1)
        
        # Read data back from 0x00(00), 4 bytes
        # status, pres MSB1, pres MSB, pres LSB
        data = self.i2c.readBlock(0x00, 4)
        
        # Convert the data to 20-bits
        pres = ((data[1] * 65536) + (data[2] * 256) + (data[3] & 0xF0)) / 16
        pressure = (pres / 4.0) / 1000.0
        
        return pressure, altitude, cTemp

    def _setSeaLevel(self, pressure):
        val = pressure // 2
        highByte = (val >> 8) & 0xFF
        lowByte = val & 0xFF
        self.i2c.writeBlock(0x14, [highByte, lowByte])

    def _getSeaLevel(self):
        # Read the sea level pressure in bars.
        data = i2c.readBlock(0x14, 2)
        # Reconstruct 16-bit value and scale back to pascals.
        pressure = (data[0] << 8) | data[1]
        return pressure * 2.0


