# oxo7seg.py
# Version 1.01 - Feb 6, 2019
# For TM1637 based 4-digit display

from machine import Pin
from time import sleep

PATTERN = b'\0\x86\x22\0\0\0\0\x02\0\0\0\0\x04\x40\x80\x52\x3f\x06\x5b\x4f\x66\x6d\x7d\x07\x7f\x6f\0\0\0\x48\0\0\x5d\x77\x7c\x39\x5e\x79\x71\x3d\x76\x30\x0e\x70\x38\x55\x54\x3f\x73\x67\x50\x2d\x78\x3e\x36\x6a\x49\x6e\x1b\x39\x64\x0f\x23\x08\x20\x77\x7c\x58\x5e\x79\x71\x3d\x74\x10\x0c\x70\x30\x55\x54\x5c\x73\x67\x50\x2d\x78\x1c\x36\x6a\x49\x6e\x1b\0\x30\0\x41'

class FourDigit:
    myData = [0,0,0,0]
    
    def __init__(self, clk = 23, dio = 5, lum = 4):
        s = self
        s.clk = clk
        s.dio = dio
        s.lum = lum
        s.colon = False
        s.startPos = 0
        s.text = None
        s.clk == clk
        s.dio == dio
        s.pclk = Pin(clk, Pin.OUT) 
        s.pdio = Pin(dio, Pin.OUT) 
        s.cw = s.pclk.value
        s.dw = s.pdio.value

    def erase(self):
        self.colon = False
        self.show("    ")

    def show(self, text, pos = 0):
        s = self
        s.startPos = pos
        s.pos = pos
        s.text = str(text)  # digits to chars
        if len(s.text) < 4:
            s.text = "%-4s" % s.text
        s._cropText()
        
    def scroll(self, text):
        s = self
        s.show(text)
        sleep(1.5)
        while s.toLeft() > 0:
            sleep(1)
        sleep(1)    
            
    def toRight(self):
        s = self
        if s.text == None:
            return -1
        s.pos -= 1    
        s._cropText()
        return max(0, 4 + s.pos)

    def toLeft(self):
        s = self
        if s.text == None:
            return -1
        s.pos += 1    
        s._cropText()
        nb = len(s.text) - s.pos
        return max(0, nb)

    def toStart(self):
        s = self
        if s.text == None:
            return -1
        s.pos = s.startPos    
        s._cropText()
       
    def setLuminosity(self, lum):        
        self.lum = lum

    def setColon(self, enable):        
        self.colon = enable

    def _cropText(self):
        s = self
        n = len(s.text)
        data = [' '] * (n + 8)
        for i in range(n):
            data[i + 4] = s.text[i]
        start = max(0, s.pos + 4)
        start = min(start, len(data) - 4)
        end = min(start + n, len(data))
        data = s._toSegment(data[start:end])
        s._prepare(0x40)    
        s._wB(0xC0)
        for i in range(4):
            s._wB(data[i])
        s._commit()
        
    def _wB(self, data):
        s = self
        for i in range(8):
            s.cw(0)
            if data & 0x01:
                s.dw(1)
            else:
                s.dw(0)
            data = data >> 1
            s.cw(1)

        s.cw(0)
        s.dw(1)
        s.cw(1)
        
        # wait for ACK
        s.pdio = Pin(self.dio, Pin.IN) 
        while s.pdio.value() == 1:
            sleep(0.001)
        s.pdio = Pin(s.dio, Pin.OUT) 
    
    def _toSegment(self, text):
        data = []
        msb = 0
        if self.colon:
            msb = 0x80
        for c in text:
            data.append(PATTERN[ord(c) - 32] + msb)
        return data

    def _start(self):
        s = self
        s.cw(1)
        s.dw(1)
        s.dw(0) 
        s.cw(0) 
    
    def _stop(self):
        s = self
        s.cw(0) 
        s.dw(0) 
        s.cw(1)
        s.dw(1)
        
    def _prepare(self, addr):        
        s = self
        s._start()
        s._wB(addr)
        s._stop()
        s._start()

    def _commit(self):
        s = self
        s._stop()
        s._start()
        s._wB(0x88 + self.lum)
        s._stop()
