# oxobutton.py
# Version 2.2 - Feb. 12, 2019

import time
import utime
from machine import Pin

BUTTON_L1 = 0
BUTTON_L2 = 32
BUTTON_L3 = 33
BUTTON_R1 = 14
BUTTON_R2 = 13
BUTTON_R3 = 27

_isRunning = False
_oldTime = utime.ticks_us()
_inCallback = False
   
# --------------- class Button ------------------------------------
class Button:
    _debounceTime = 200000 # us
    def __init__(self, pinNumber, buttonClick = None):
        self._pinNumber = pinNumber
        self._pressed = False
        self._buttonClick = buttonClick
        if pinNumber == 0:
            self._pin = Pin(pinNumber, Pin.IN, Pin.PULL_UP)
            self._pin.irq(trigger = Pin.IRQ_FALLING | Pin.IRQ_RISING, handler = self.buttonCallback)
        else:
            self._pin = Pin(pinNumber, Pin.IN, Pin.PULL_DOWN)
            self._pin.irq(trigger = Pin.IRQ_FALLING | Pin.IRQ_RISING, handler = self.buttonCallback)
        
    def wasPressed(self):
        time.sleep(0.01)  # in case of narrow loops
        if self._pressed:
            self._pressed = False
            return True
        return False
    
    def isPressed(self):
        time.sleep(0.01)  # in case of narrow loops
        if self._pinNumber == BUTTON_L1:
            return self._pin.value() == 0
        return self._pin.value() == 1

    def buttonCallback(self, p):
        global _isRunning, _oldTime, _inCallback
        if _inCallback:
            return
        _inCallback = True
        if _isRunning:
            if utime.ticks_us() - _oldTime > Button._debounceTime:
                _isRunning = False
            else:
                _inCallback = False    
                return
        _oldTime = utime.ticks_us()
        _isRunning = True
        if self._pinNumber == BUTTON_L1:
            if p.value() == 1:
                if self._buttonClick != None:
                    self._buttonClick(self._pinNumber)
            else:
                self._pressed = True
        else:
            if p.value() == 0:
                if self._buttonClick != None:
                    self._buttonClick(self._pinNumber)
            else:
                self._pressed = True
        _inCallback = False    