# oxocard.py
# Version 1.05 - Feb 19, 2019

from time import sleep
from random import randint
from math import sin, cos, radians, pi
from pixfont import font, font3x5
import gc
from globals import *
from globals import __np

_pList = []
def getRandomPos():
    global _pList
    if len(_pList) == 0:
        _pList = [n for n in range(64)]
    r = randint(0, len(_pList) - 1)    
    m = _pList[r]
    del _pList[r]
    return m % 8, m // 8

# ---------------------- class OxoGrid -----------------------------------
class OxoGrid:
    _np = None
    oxocard = True
    _isRepaintEnabled = True
    _instance = None
    
    def __init__(self):
        self._dimFactor = 1
        self._w = 24
        self._h = 24
        self._origin = [8, 8] # default position   
        self._buf = [[0 for k in range(self._w)] for i in range(self._h)]     
        OxoGrid._np = __np
        OxoGrid._instance = self

    def clear(self, *args):
        if len(args) == 0:
            self.clear(0)
            return
        for i in range(0, self._h):
            for k in range(0, self._w):
                self._buf[i][k] = self._toColorInt(args[0])
        if OxoGrid._isRepaintEnabled:
            self._repaint()

    def setOrigin(self, *args):
        self._origin = self._toPoint(*args)
        if OxoGrid._isRepaintEnabled:
            self._repaint()

    def getOrigin(self, *args):
        return self._origin[:]

    def dim(self, factor):
        self._dimFactor = factor
        if OxoGrid._isRepaintEnabled:
            self._repaint()

    @staticmethod
    def enableRepaint(enable):
        OxoGrid._isRepaintEnabled = enable

    @staticmethod
    def repaint():
        if OxoGrid._instance != None:
            OxoGrid._instance._repaint()

    def _repaint(self):
        sleep(0.01) # let the process yield, otherwise watchdog is complaining
        for i in range(0, 8):
            for k in range(0, 8):
                m = i + self._origin[1]
                n = k + self._origin[0]
                if OxoGrid.oxocard:
                    index = 8 * i + k + 1
                else:
                    if i % 2 == 0:
                        index = 65 - (8 * i + k + 1)
                    else:
                        index = 65 - (8 * i + 8 - k)
                if 0 <= m < self._h and 0 <= n < self._w:
                    OxoGrid._np.set(index, OxoGrid._reduce(self._buf[m][n], self._dimFactor), update = False) 
                else:
                    OxoGrid._np.set(index, 0, update = False) 
        OxoGrid._np.show()

    def dot(self, x, y, c):
        self._toBuf(x, y, self._toColorInt(c))
        if OxoGrid._isRepaintEnabled:
            self._repaint()
            
    def getColor(self, *args):
        return intToRGB(self.getColorInt(*args))
    
    def getColorInt(self, *args):
        pt = self._toPoint(*args)
        i = pt[0] + self._origin[0]
        k = pt[1] + self._origin[1]
        if 0 <= i < self._w and 0 <= k < self._h:
            return self._buf[k][i]
        return 0        
    
    def image(self, matrix):
        for y in range(len(matrix)):
            for x in range(len(matrix[0])):
                self._toBuf(x, y, self._toColorInt(matrix[y][x]))
        if OxoGrid._isRepaintEnabled:
            self._repaint()

    def getArray(self):
        buf = []
        for i in range(self._h):
           line = self._buf[i][:] 
           buf.append(line)
        return tuple(buf)
    
    def insertBigChar(self, char, charColor, bgColor):
        self.image(self._bigCharToMatrix(char, charColor, bgColor))

    def bigTextScroll(self, text, textColor = (255, 255, 255), bgColor = (0, 0, 0), speed = 10):
        self._textScroll(text, textColor, bgColor, speed, False)
        
    def smallTextScroll(self, text, textColor = (255, 255, 255), bgColor = (0, 0, 0), speed = 5):
        self._textScroll(text, textColor, bgColor, speed, True)
        
    def display(self, chars, textColor = (255, 255, 255), bgColor = (0, 0, 0)):
        self._insertSmallCharsWithSign(chars, textColor, bgColor)     

    # --------------- static methods ------------------    
    @staticmethod
    def _reduce(c, r):
        if c == 0:
            return 0
        blue = ((c & 255) + 1) // 2
        if blue > 0:
            blue =  max(1, int(blue / r + 0.5))
        green = (((c >> 8) & 255) + 1) // 2
        if green > 0:
            green = max(1, int(green / r + 0.5))
        red = (((c >> 16) & 255) + 1) // 2
        if red > 0:    
            red = max(1, int(red / r + 0.5))
        return (red << 16) + (green << 8) + blue

    def dispose(self):
        if OxoGrid._instance != None:
            OxoGrid._np = None    
            self._buf = None
            OxoGrid._instance = None
            gc.collect()

    # --------------- private methods ------------------    
    def _toBuf(self, x, y, c):
        i = x + self._origin[0]
        k = y + self._origin[1]
        if 0 <= i < self._w and 0 <= k < self._h:
            self._buf[k][i] = c

    def _toColorInt(self, c):
        if type(c) in [list, tuple] and len(c) == 3:
            return (c[0] << 16) + (c[1] << 8) + c[2]
        return c

    def _toPoint(self, *args):
        if len(args) == 2:
            return (args[0], args[1])
        return args[:]
   
    def _bigCharToMatrix(self, char, charColor, bgColor):                    
        if type(char) == int:
            char = str(char)
        matrix = []
        a = font[ord(char) - 33]
        for i in range(8):
            hex = a[i]
            matrix.append(self._hexToLine(hex, charColor, bgColor))
        return matrix
 
    def _smallCharsToMatrix(self, chars, charColor, bgColor):
        chars = str(chars.upper())
        a = [0] * 2
        n = min(len(chars), 2)
        for i in range(n):
            k = ord(chars[i]) - 32
            if k >= 0 and k < len(font3x5):
                a[i] = font3x5[k]
            else:
                a[i] = 0
                
        bgC = rgbToInt(bgColor)
        digit0 = self._toDigit(a[0], charColor, bgColor)
        digit1 = self._toDigit(a[1], charColor, bgColor)
        matrix = [[bgC for k in range(8)] for i in range(8)]
        for i in range(5):
            for k in range(3):
                matrix[i + 1][k] = digit0[i][k] 
                matrix[i + 1][k + 4] = digit1[i][k]
        return matrix

    def _textScroll(self, text, textColor, bgColor, speed, small):
        if len(text) == 0:
            return
        isRepaint = OxoGrid._isRepaintEnabled
        OxoGrid._isRepaintEnabled = True
        if len(text) == 1:
            if small:
                self.display(text, textColor, bgColor)
            else:
                self.insertBigChar(text, textColor, bgColor)
        elif len(text) == 2 and small:
            self.display(text, textColor, bgColor)
        else:
            self._scrollBuf = [None] * 8 # 8 lines with 16 columns
            speed = min(12, speed)
            speed = max(1, speed)
            dt = (10**((14 - speed) / 4)) / 1000
            if small:
                if len(text) % 2 == 1:
                    text = " " + text
                m1 = self._smallCharsToMatrix(text[0] + text[1], textColor, bgColor)
                m2 = self._smallCharsToMatrix(text[2] + text[3], textColor, bgColor)
            else:
                m1 = self._bigCharToMatrix(text[0], textColor, bgColor)
                m2 = self._bigCharToMatrix(text[1], textColor, bgColor)
            n = 1
            while True:
                for i in range(8):
                    self._scrollBuf[i] = m1[i] + m2[i]
                for _ in range(8):    
                    self._showScrollBuf()
                    sleep(dt)
                    self._shiftLeft()
                m1 = m2
                n += 1
                if small:    
                    if n == len(text) // 2:
                        break
                    m2 = self._smallCharsToMatrix(text[2 * n] + text[2 * n + 1], textColor, bgColor)
                else:
                    if n == len(text):
                        break
                    m2 = self._bigCharToMatrix(text[n], textColor, bgColor)
        OxoGrid._isRepaintEnabled = isRepaint
    
    def _toDigit(self, intCode, charColor, bgColor):
        digitList = [[bgColor for k in range(3)] for i in range(5)]
        mask = 1
        for i in range(5):
            for k in range(3):
                if mask & intCode != 0:
                    digitList[i][k] = charColor
                mask *= 2
        return digitList    
    
    def _hexToLine(self, hex, textColor, bgColor):
        line = [bgColor] * 8
        mask = 1
        for k in range(8):
            line[k] = textColor if ((hex & mask) != 0) else bgColor
            mask = 2 * mask
        return line

    def _hexToLine1(self, hex, textColor, bgColor):
        line = [bgColor] * 3
        mask = 4
        for k in range(3):
            line[k] = textColor if ((hex & mask) != 0) else bgColor
            mask = mask // 2
        return line

    def _shiftLeft(self):
        for i in range(8):  # rows
            a = self._scrollBuf[i][:]
            a =  a[1:] + [0]
            for k in range(16):
                self._scrollBuf[i][k] = a[k]

    def _showScrollBuf(self):
        matrix = [0] * 8
        for i in range(8):
            matrix[i] = self._scrollBuf[i][0:8]
        self.image(matrix)

    def _insertSmallCharsWithSign(self, chars, charColor, bgColor):
        sign = False
        if type(chars) == float:
            chars = int(chars)
        if type(chars) == int and chars < 0:
            sign = True
            chars = -chars
        isEnabled = OxoGrid._isRepaintEnabled
        OxoGrid._isRepaintEnabled = False
        self.image(self._smallCharsToMatrix(str(chars), charColor, bgColor))
        if sign:
            for x in range(3):
                self.dot(x, 7, charColor)
        OxoGrid._isRepaintEnabled = isEnabled
        if OxoGrid._isRepaintEnabled:
            self._repaint()

# ------------ global wrapper -------------------      
_grid = OxoGrid()

def dot(x, y, color):
    _grid.dot(x, y, color)

def dispose():
    _grid.dispose()

def clear(*args):
    _grid.clear(*args)

def setOrigin(*args):
    _grid.setOrigin(*args)

def getOrigin():
    return _grid.getOrigin()
    
def dim(dimFactor):
    _grid.dim(dimFactor)    

def getColor(*args):
    return _grid.getColor(*args)    

def getColorInt(*args):
    return _grid.getColorInt(*args)    

def image(matrix, *args):
    _grid.image(matrix, *args)

def insertBigChar(char, charColor = (255, 255, 255), bgColor = (0, 0, 0)):
    _grid.insertBigChar(char, charColor, bgColor)
    
def display(chars, charColor = (255, 255, 255), bgColor = (0, 0, 0)):
    _grid.display(chars, charColor, bgColor)     

def smallTextScroll(text, textColor = (255, 255, 255), bgColor = (0, 0, 0), speed = 5):
    _grid.smallTextScroll(text, textColor, bgColor, speed)

def bigTextScroll(text, textColor = (255, 255, 255), bgColor = (0, 0, 0), speed = 10):
    _grid.bigTextScroll(text, textColor, bgColor, speed)
    
def getArray():
    return _grid.getArray()

def enableRepaint(enable):
    OxoGrid.enableRepaint(enable)

def repaint():
    OxoGrid.repaint()
