# oxocardext.py
# Version 1.04 - Jan 8, 2019

from oxocard import *

class OxoGridExt(OxoGrid):
    def clearWindow(self, *args):
        if len(args) == 0:
            self.clearWindow(0)
            return
        for i in range(0, 8):
            for k in range(0, 8):
                m = i + self._origin[1]
                n = k + self._origin[0]
                self._buf[m][n] = self._toColorInt(args[0])
        if OxoGrid._isRepaintEnabled:
            self._repaint()
           
    def clearFrame(self, frameNumber, *args):
        if len(args) == 0:
            self.clearFrame(frameNumber, 0)
            return
        for i in range(0, 8):
            for k in range(0, 8):
                m = i + self._frameToOrigin(frameNumber)[1]
                n = k + self._frameToOrigin(frameNumber)[0]
                self._buf[m][n] = self._toColorInt(args[0])
        if OxoGrid._isRepaintEnabled:
            self._repaint()

    def setFrame(self, frameNumber):
        self._origin = self._frameToOrigin(frameNumber)
        if OxoGrid._isRepaintEnabled:
            self._repaint()
  
    def translate(self, vector):
        tmp = [[0 for k in range(self._w)] for i in range(self._h)]
        for i in range(self._h):
            for k in range(self._w):
                x = k - vector[0]
                y = i - vector[1]
                if 0 <= x < self._w and 0 <= y < self._h:
                    tmp[i][k] = self._buf[y][x]
                else:
                    tmp[i][k] = 0
        self._buf = tmp
        if OxoGrid._isRepaintEnabled:
            self._repaint()

    def rotate(self, centerx, centery, angle):
        centerx += self._origin[0]
        centery += self._origin[1]
        angle= radians(-angle)
        tmp = [[0 for k in range(self._w)] for i in range(self._h)]
        for i in range(self._h):
            for k in range(self._w):
                x = k - centerx
                y = i - centery
                x1 = int(round(cos(angle) * x - sin(angle) * y) + centerx)
                y1 = int(round(sin(angle) * x + cos(angle) * y) + centery)
                if 0 <= x1 < self._w and 0 <= y1 < self._h:
                    tmp[i][k] = self._buf[y1][x1]
                else:
                    tmp[i][k] = 0
        self._buf = tmp
        if OxoGrid._isRepaintEnabled:
            self._repaint()

    def line(self, x, y, dir, length, c):
        if length <= 0:
            return
        enabled =  OxoGrid._isRepaintEnabled
        OxoGrid._isRepaintEnabled = False
        dir = dir % 8
        count = 0
        while True:
            self.dot(x, y, c)
            count += 1
            if count == length:
                break
            if dir == 0:
                x += 1
            elif dir == 1:
                x += 1
                y -= 1
            elif dir == 2:
                y -= 1
            elif dir == 3:
                x -= 1
                y -= 1
            elif dir == 4:
                x -= 1
            elif dir == 5:
                x -= 1
                y += 1
            elif dir == 6:
                y += 1
            elif dir == 7:
                x += 1
                y += 1
        OxoGrid._isRepaintEnabled = enabled
        if enabled:        
            self._repaint()

    def arrow(self, x, y, dir, length, c):
        if length <= 0 or dir < 0:
            return
        enabled =  OxoGrid._isRepaintEnabled
        OxoGrid._isRepaintEnabled = False
        dir = dir % 8
        self.line(x, y, dir, length, c)
        a = pi / 4 * dir
        if dir % 2 == 0:
            r = length - 1
        else:
            r = 1.41 * (length - 1)
        x1 = int(round(x + r * cos(-a)))
        y1 = int(round(y + r * sin(-a)))
        if dir == 0:
            p1 = x1 - 1, y - 1
            p2 = x1 - 1, y + 1
        elif dir == 1:
            p1 = x1 - 1, y1
            p2 = x1, y1 + 1
        elif dir == 2:
            p1 = x1 - 1, y1 + 1
            p2 = x1 + 1, y1 + 1
        elif dir == 3:
            p1 = x1, y1 + 1
            p2 = x1 + 1, y1 
        elif dir == 4:
            p1 = x1 + 1, y1 + 1
            p2 = x1 + 1, y1 - 1
        elif dir == 5:
            p1 = x1 + 1, y1
            p2 = x1, y1 - 1
        elif dir == 6:
            p1 = x1 + 1, y1 - 1
            p2 = x1 - 1, y1 - 1
        elif dir == 7:
            p1 = x1, y1 - 1
            p2 = x1 - 1, y1 
        self.dot(p1[0], p1[1], c)
        self.dot(p2[0], p2[1], c)        
        OxoGrid._isRepaintEnabled = enabled
        if enabled:        
            self._repaint()
        
    def rectangle(self, ulx, uly, w, h, c):
        c = self._toColorInt(c)
        for x in range(ulx, ulx + w):
            self._toBuf(x, uly, c)
            self._toBuf(x, uly + h - 1, c)
        for y in range(uly, uly + h):
            self._toBuf(ulx, y, c)
            self._toBuf(ulx + w - 1, y, c)
        if OxoGrid._isRepaintEnabled:
            self._repaint()

    def fillRectangle(self, ulx, uly, w, h, c):
        c = self._toColorInt(c)
        for y in range(uly, uly + h):
            for x in range(ulx, ulx + w):
                self._toBuf(x, y, c)
        if OxoGrid._isRepaintEnabled:
            self._repaint()

    def circle(self, xcenter, ycenter, r, c):
        d = -r
        x = r
        y = 0
        while True:
            for a in [-x, x]:
               for b in [-y, y]:
                    self._toBuf(a + xcenter, b + ycenter, self._toColorInt(c))
                    self._toBuf(b + xcenter, a + ycenter, self._toColorInt(c))
            d = d + 2 * y + 1
            y = y + 1
            if d > 0:
                d = d - 2 * x + 2
                x = x - 1
            if y > x:
                break
        if OxoGrid._isRepaintEnabled:
            self._repaint()
    
    def fillCircle(self, xcenter, ycenter, r, c):
        b = OxoGrid._isRepaintEnabled 
        OxoGrid._isRepaintEnabled = False
        for z in range(0, r + 1):
            self.circle(xcenter, ycenter, z, c)
        OxoGrid._isRepaintEnabled = b
        if OxoGrid._isRepaintEnabled:
            self._repaint()

    def _frameToOrigin(self, frameNumber):
        x = (frameNumber % 3) * 8
        y = (frameNumber // 3) * 8
        return (x, y)


# ------------ global wrapper -------------------      
_grid = OxoGridExt()

def dot(x, y, color):
    _grid.dot(x, y, color)

def dispose():
    _grid.dispose()

def clear(*args):
    _grid.clear(*args)
    
def clearWindow(*args):
    _grid.clearWindow(*args)
    
def clearFrame(frame, *args):
    _grid.clearFrame(*args)
    
def setOrigin(*args):
    _grid.setOrigin(*args)

def getOrigin():
    return _grid.getOrigin()

def setFrame(frameNumber):
    _grid.setFrame(frameNumber)

def dim(dimFactor):
    _grid.dim(dimFactor)    

def getColor(*args):
    return _grid.getColor(*args)    

def getColorInt(*args):
    return _grid.getColorInt(*args)    

def line(x, y, dir, length, color):
    _grid.line(x, y, dir, length, color)

def arrow(x, y, dir, length, color):
    _grid.arrow(x, y, dir, length, color)

def rectangle(ulx, uly, w, h, color):
    _grid.rectangle(ulx, uly, w, h, color)

def fillRectangle(ulx, uly, w, h, color):
    _grid.fillRectangle(ulx, uly, w, h, color)

def circle(xcenter, ycenter, r, color):
    _grid.circle(xcenter, ycenter, r, color)

def fillCircle(xcenter, ycenter, r, color):
    _grid.fillCircle(xcenter, ycenter, r, color)

def image(matrix, *args):
    _grid.image(matrix, *args)

def translate(vector):
    _grid.translate(vector)

def rotate(centerx, centery, angle):   
    _grid.rotate(centerx, centery, angle)

def insertBigChar(char, charColor = (255, 255, 255), bgColor = (0, 0, 0)):
    _grid.insertBigChar(char, charColor, bgColor)
    
def display(chars, charColor = (255, 255, 255), bgColor = (0, 0, 0)):
    _grid.display(chars, charColor, bgColor)     

def smallTextScroll(text, textColor = (255, 255, 255), bgColor = (0, 0, 0), speed = 5):
    _grid.smallTextScroll(text, textColor, bgColor, speed)

def bigTextScroll(text, textColor = (255, 255, 255), bgColor = (0, 0, 0), speed = 10):
    _grid.bigTextScroll(text, textColor, bgColor, speed)
    
def getArray():
    return _grid.getArray()

def enableRepaint(enable):
    OxoGrid.enableRepaint(enable)

def repaint():
    OxoGrid.repaint()
