# oxorover.py
# A: right motor, B: left motor
# Version 2.02, Feb. 4, 2019

from machine import Pin, PWM
from time import sleep

pA1 = Pin(5)
pA2 = Pin(23)
pB1 = Pin(18)
pB2 = Pin(19)
fPWM = 50  

motA1 = PWM(pA1, freq = fPWM, duty = 0)
motA2 = PWM(pA2, freq = fPWM, duty = 0)
motB1 = PWM(pB1, freq = fPWM, duty = 0)
motB2 = PWM(pB2, freq = fPWM, duty = 0)

# ----------------------------------- class Motor -------------------------
class Motor:
    def __init__(self, label, speed = 400):
        self.label = label
        self.speed = speed

    def setSpeed(self, speed):                                                                            
        self.speed = speed

    def getSpeed(self):                                                                            
        return self.speed
    
    def forward(self):
        if self.label == "A":
            motA1.duty(self.speed)    
            motA2.duty(0)    
        if self.label == "B":
            motB1.duty(self.speed)    
            motB2.duty(0)    

    def backward(self):
        if self.label == "A":
            motA1.duty(0)    
            motA2.duty(self.speed)    
        if self.label == "B":
            motB1.duty(0)    
            motB2.duty(self.speed)    

    def stop(self, brake = False):
        if brake:
            d = 1023
        else:
            d = 0 
        if self.label == "A":    
            motA1.duty(d)    
            motA2.duty(d)    
        if self.label == "B":    
            motB1.duty(d)    
            motB2.duty(d)    
            
# ----------------------------------- class Gear -------------------------
class Gear:
    def __init__(self, speed = 400):
        self.motA = Motor("A")
        self.motB = Motor("B")
        self.speed = speed
        self.motA.setSpeed(speed)
        self.motB.setSpeed(speed)
        
    def setSpeed(self, speed):
        self.speed = speed
            
    def getSpeed(self):
        return self.speed
            
    def forward(self):
        self.motA.setSpeed(self.speed)
        self.motB.setSpeed(self.speed)
        self.motA.forward()    
        self.motB.forward()    

    def backward(self):
        self.motA.setSpeed(self.speed)
        self.motB.setSpeed(self.speed)
        self.motA.backward()    
        self.motB.backward()    
        
    def left(self):
        self.motA.setSpeed(self.speed)
        self.motB.setSpeed(self.speed)
        self.motA.forward()
        self.motB.backward()

    def right(self):
        self.motA.setSpeed(self.speed)
        self.motB.setSpeed(self.speed)
        self.motA.backward()
        self.motB.forward()

    def leftArc(self, radius):
        axeLength = 10
        if radius < axeLength:
            speed1 = 0
        else:            
            speed1 = int(self.speed * (radius - axeLength) / (2 * (radius + axeLength)))
        self.motA.setSpeed(self.speed)
        self.motB.setSpeed(speed1)
        self.motA.forward()    
        self.motB.forward()    

    def rightArc(self, radius):
        axeLength = 10
        if radius < axeLength:
            speed1 = 0
        else:            
            speed1 = int(self.speed * (radius - axeLength) / (2 * (radius + axeLength)))
        self.motA.setSpeed(speed1)
        self.motB.setSpeed(self.speed)
        self.motA.forward()    
        self.motB.forward()    

    def stop(self, brake = False):
        self.motA.stop(brake)    
        self.motB.stop(brake)
 

