# pwmsound.py
# Version 1.01, Jan. 27, 2019

from machine import PWM, Pin
import time

class PWMSound:
    ENTERTAINER = ('d', 'd#', 'e', 'c2:2', 'e', 'c2:3', 'e', 'c2:3', 'c2', 'd2',
                   'd2#', 'e2', 'c2', 'd2', 'e2:2', 'h', 'd2:2', 'c2:4')
    
    RINGTONE = ('c', 'd', 'e:2', 'g', 'd', 'e', 'f:2', 'a', 'e', 'f', 'g:2', 'h', 'c2:4')
    
    BIRTHDAY = ('c:3', 'c', 'd:4', 'c:4', 'f:4', 'e:8', 'c:3', 'c', 'd:4', 'c:4', 'g:4', 
                'f:8', 'c:3', 'c:1', 'c2:4', 'a:4', 'f:4', 'e:4', 'd:4', 'a#:3', 'a#:1', 'a:4', 'f:4', 'g:4', 'f:8')
    
    JUMP_UP = ('c2', 'd2', 'e2', 'f2', 'g2')
    
    JUMP_DOWN = ('g2', 'f2', 'e2', 'd2', 'c2')
    
    DADADADUM = ('r:2', 'g', 'g', 'g', 'd#:8', 'r:2', 'f', 'f', 'f', 'd:8')
    
    POWER_UP = ('g', 'c2', 'e', 'g:2', 'e', 'g:3')
    
    POWER_DOWN = ('g2', 'd#', 'c', 'g:2', 'h', 'c2:3')
    
    PUNCHLINE = ('c2:3', 'g', 'f#', 'g', 'g#:3', 'g', 'r', 'h:2', 'c2')
    
    WEDDING = ('c:4', 'f:3', 'f', 'f:8', 'c:4', 'g:3', 'e', 'f:8', 'c:4', 'f:3',
               'a', 'c2:4', 'a:3', 'f', 'f:4', 'e:3', 'f', 'g:8')
    
    BOOGYWOODY = ('c', 'e', 'g', 'a', 'a#', 'a', 'g', 'e', 'c', 'e', 'g', 
                  'a', 'a#', 'a', 'g', 'e', 'f', 'a', 'c2', 'd2', 'd2#', 'd2', 'c2', 'a', 
                  'c', 'e', 'g', 'a', 'a#', 'a', 'g', 'e', 'g', 'h', 'd2', 'e2', 'f2', 'e2', 'd2', 'h',
                  'c', 'e', 'g', 'a', 'a#', 'a', 'g', 'e', 'c:2')
    
    PRELUDE = ('c', 'e', 'g', 'c2', 'e2', 'g', 'c2', 'e2', 'c', 'e', 'g',
               'c2', 'e2', 'g', 'c2', 'e2', 'c', 'd', 'g', 'd2', 'f2', 'g', 'd2', 
               'f2', 'c', 'd', 'g', 'd2', 'f2', 'g', 'd2', 'f2', 'H', 'd', 'g', 'd2',
               'f', 'g', 'd2', 'f2', 'H', 'd', 'g', 'd2', 'f2', 'g', 'd2', 'f2', 'c', 
               'e', 'g', 'c2', 'e2', 'g', 'c2', 'e2', 'c', 'e', 'g', 'c2', 'e2', 'g', 'c2', 'e2')
    
    
    _notes = {'r':0, 'C':131, 'C#':139, 'D':147, 'D#':156, 'E':165, 'F':175, 'F#':185, 
             'G':196, 'G#':208, 'A':220, 'A#':233, 'H':247, 
             'c':262, 'c#':277, 'd':294, 'd#':313, 'e':330, 'f':349, 'f#':370, 
             'g':392, 'g#':415, 'a':440, 'a#':466, 'h':494, 
             'c2':523, 'c2#':524, 'd2':587, 'd2#':622, 'e2':659, 'f2':698, 'f2#':740, 
             'g2':784, 'g2#':831, 'a2':880, 'a2#':932, 'h2':988, 
             'c3':1046}
   
    def __init__(self, port = 19):
        p = Pin(port)
        self.pwm = PWM(p, freq = 5, duty = 0)  

    def playTone(self, freq, duration):
        self.pwm.duty(512)
        if type(freq) == str:
            index = freq.find(':')
            if index == -1: 
                freq = PWMSound._notes[freq]
            else:
                duration = duration * int(freq[index + 1:])
                freq = PWMSound._notes[freq[0:index]]
        self.pwm.freq(freq)
        duration = duration / 1000 # ms to s        
        time.sleep(duration)
        self.pwm.duty(0)   
       
    def playSong(self, song, duration = 170):
        for tone in song:
            self.playTone(tone, duration) 
            time.sleep(0.03)
