# sht31.py
# Sensirion temperature/humidity sensor
# Version 2.00 - Aug 14, 2018

from utils import I2C_com
from machine import Pin
import time

# ------------------------------ class SHT31 -----------------------------------
# Sensirion temperature/humidity sensor
class SHT31:
    R_HIGH   = const(1)
    R_MEDIUM = const(2)
    R_LOW    = const(3)
    
    _map_cs_r = {
        True: {
            R_HIGH : b'\x2c\x06',
            R_MEDIUM : b'\x2c\x0d',
            R_LOW: b'\x2c\x10'
            },
        False: {
            R_HIGH : b'\x24\x00',
            R_MEDIUM : b'\x24\x0b',
            R_LOW: b'\x24\x16'
            }
        }

    def __init__(self, scl = Pin(22), sda = Pin(21), addr = 0x44):
        self._i2c_com = I2C_com(addr, scl, sda)
        self._addr = addr

    def _send(self, buf):
        self._i2c_com.writeTo(buf)

    def _recv(self, count):
        return self._i2c_com.readFrom(count)

    def _raw_temp_humi(self, r=R_HIGH, cs=True):
        """
        Read the raw temperature and humidity from the sensor and skips CRC
        checking.
        Returns a tuple for both values in that order.
        """
        if r not in (R_HIGH, R_MEDIUM, R_LOW):
            raise ValueError('Wrong repeatabillity value given!')
        self._send(self._map_cs_r[cs][r])
        time.sleep_ms(50)
        raw = self._recv(6)
        return (raw[0] << 8) + raw[1], (raw[3] << 8) + raw[4]

    def get_temp_humi(self, resolution=R_HIGH, clock_stretch=True, celsius=True):
        """
        Read the temperature in degree celsius or fahrenheit and relative
        humidity. Resolution and clock stretching can be specified.
        Returns a tuple for both values in that order.
        """
        t, h = self._raw_temp_humi(resolution, clock_stretch)
        if celsius:
            temp = -45 + (175 * (t / 65535))
        else:
            temp = -49 + (315 * (t / 65535))
        return temp, 100 * (h / 65535)
