# weather.py
# Version 1.02, Sept. 11, 2018

from tcpcom import *
import ujson
import utime
import utils

def _replace_umlaute(s):
    di = {"\xE4" : "ae", "\xF6" : "oe", "\xFC" : "ue",
          "\xC4" : "Ae", "\xD6" : "Oe", "\xDC" : "Ue",
          "\xDF" : "ss"}
    for key in di:
        s = s.replace(key, di[key])      
    return s

def _errorDict(status):
    return {'status' : status, 
            'temp' : 0, 
            'pressure' : 0,
            'humidity' : 0, 
            'temp_min' : 0, 
            'temp_max' : 0, 
            'description' : '-'}

def _getValues(data):
    print('Data response:')
    print(data)
    data_dict = ujson.loads(data)
    code = data_dict['cod']
    if code == 401:  # int
        return _errorDict('Invalid authorization key')
    if code == "404":  # string!
        return _errorDict('City not found')
    main = data_dict['main']
    status = 'OK'
    temp = main['temp']
    pressure = main['pressure']
    humidity = main['humidity']
    temp_min = main['temp_min']
    temp_max = main['temp_max']
    weather = data_dict['weather']
    weather_dict = weather[0]
    description = _replace_umlaute(weather_dict['description'])
    sys = data_dict['sys']
    sunrise = utils.getTime(sys['sunrise'])
    sunset = utils.getTime(sys['sunset'])
    datetime = utils.getDateTime(data_dict['dt'])
    return {'status' : status, 
            'temp' : utils.toCentigrades(temp), 
            'pressure' : pressure, 
            'humidity' : humidity, 
            'temp_min' : utils.toCentigrades(temp_min), 
            'temp_max' : utils.toCentigrades(temp_max), 
            'description' : description,
            'sunrise' : sunrise,
            'sunset' : sunset,
            'datetime' : datetime}

def request(ssid, password, city, key = '3344d39b56d7bf351f39ddb8a84961a4'):
    if not Wlan.connect(ssid, password):
        return _errorDict('Connection to AP failed')
    client = HTTPClient()
    rc = client.connect(_host, _port)
    if not rc:
        return _errorDict('Connection to ' + _host + 'failed')
    param = '/data/2.5/weather?q=' + city + '&appid=' + key + '&lang=de' # de for description
    response = client.sendGetRequest(param)
    message_part = response[response.find('\r\n\r\n') + 4:]
    client.closeConnection()
    values = _getValues(message_part)
    return values

_host = 'api.openweathermap.org'
_port = 80
